@@ -89,7 +89,6 @@
 #define SF_IGNORE_PRST	0x00080000	/* ignore persistence */
 
 #define SF_SRV_REUSED   0x00100000	/* the server-side connection was reused */
-#define SF_SRV_REUSED_ANTICIPATED  0x00200000  /* the connection was reused but the mux is not ready yet */
 
 
 /* flags for the proxy of the master CLI */
@@ -140,12 +139,12 @@ struct stream {
 	int16_t priority_class;         /* priority class of the stream for the pending queue */
 	int32_t priority_offset;        /* priority offset of the stream for the pending queue */
 
-	struct list list;               /* position in global streams list */
-	struct list by_srv;             /* position in server stream list */
+	struct list list;               /* position in the thread's streams list */
+	struct mt_list by_srv;          /* position in server stream list */
 	struct list back_refs;          /* list of users tracking this stream */
 	struct buffer_wait buffer_wait; /* position in the list of objects waiting for a buffer */
 
-	struct freq_ctr call_rate;      /* stream task call rate without making progress */
+	struct freq_ctr call_rate;      /* stream task call rate */
 
 	short store_count;
 	enum obj_type obj_type;         /* object type == OBJ_TYPE_STREAM */
@@ -181,6 +180,9 @@ struct stream {
 	/* These two pointers are used to resume the execution of the rule lists. */
 	struct list *current_rule_list;         /* this is used to store the current executed rule list. */
 	void *current_rule;                     /* this is used to store the current rule to be resumed. */
+	int rules_exp;                          /* expiration date for current rules execution */
+
+	unsigned int stream_epoch;              /* copy of stream_epoch when the stream was created */
 	struct hlua *hlua;                      /* lua runtime context */
 
 	/* Context */
