@@ -242,15 +242,6 @@ static inline int si_rx_blocked(const struct stream_interface *si)
 	return !!(si->flags & SI_FL_RXBLK_ANY);
 }
 
-
-/* Returns non-zero if the stream interface's Rx path is blocked because of lack
- * of room in the input buffer.
- */
-static inline int si_rx_blocked_room(const struct stream_interface *si)
-{
-	return !!(si->flags & SI_FL_RXBLK_ROOM);
-}
-
 /* Returns non-zero if the stream interface's endpoint is ready to receive */
 static inline int si_rx_endp_ready(const struct stream_interface *si)
 {
@@ -381,7 +372,7 @@ static inline struct conn_stream *si_alloc_cs(struct stream_interface *si, struc
 
 	si_release_endpoint(si);
 
-	cs = cs_new(conn);
+	cs = cs_new(conn, conn->target);
 	if (cs)
 		si_attach_cs(si, cs);
 
