@@ -53,6 +53,7 @@
 #include <haproxy/sample-t.h>
 #include <haproxy/server.h>
 #include <haproxy/session.h>
+#include <haproxy/sock.h>
 #include <haproxy/stats-t.h>
 #include <haproxy/stream.h>
 #include <haproxy/stream_interface.h>
@@ -65,6 +66,7 @@
 #define PAYLOAD_PATTERN "<<"
 
 static struct applet cli_applet;
+static struct applet mcli_applet;
 
 static const char stats_sock_usage_msg[] =
 	"Unknown command. Please enter one of the following commands only :\n"
@@ -109,11 +111,11 @@ static char *cli_gen_usage_msg(struct appctx *appctx)
 		while (kw->str_kw[0]) {
 
 			/* in a worker or normal process, don't display master only commands */
-			if (master == 0 && (kw->level & ACCESS_MASTER_ONLY))
+			if (appctx->applet == &cli_applet && (kw->level & ACCESS_MASTER_ONLY))
 				goto next_kw;
 
 			/* in master don't displays if we don't have the master bits */
-			if (master == 1 && !(kw->level & (ACCESS_MASTER_ONLY|ACCESS_MASTER)))
+			if (appctx->applet == &mcli_applet && !(kw->level & (ACCESS_MASTER_ONLY|ACCESS_MASTER)))
 				goto next_kw;
 
 			/* only show expert commands in expert mode */
@@ -434,25 +436,25 @@ int listeners_setenv(struct proxy *frontend, const char *varname)
 				if (trash->data)
 					chunk_appendf(trash, ";");
 
-				if (l->addr.ss_family == AF_UNIX) {
+				if (l->rx.addr.ss_family == AF_UNIX) {
 					const struct sockaddr_un *un;
 
-					un = (struct sockaddr_un *)&l->addr;
+					un = (struct sockaddr_un *)&l->rx.addr;
 					if (un->sun_path[0] == '\0') {
 						chunk_appendf(trash, "abns@%s", un->sun_path+1);
 					} else {
 						chunk_appendf(trash, "unix@%s", un->sun_path);
 					}
-				} else if (l->addr.ss_family == AF_INET) {
-					addr_to_str(&l->addr, addr, sizeof(addr));
-					port_to_str(&l->addr, port, sizeof(port));
+				} else if (l->rx.addr.ss_family == AF_INET) {
+					addr_to_str(&l->rx.addr, addr, sizeof(addr));
+					port_to_str(&l->rx.addr, port, sizeof(port));
 					chunk_appendf(trash, "ipv4@%s:%s", addr, port);
-				} else if (l->addr.ss_family == AF_INET6) {
-					addr_to_str(&l->addr, addr, sizeof(addr));
-					port_to_str(&l->addr, port, sizeof(port));
+				} else if (l->rx.addr.ss_family == AF_INET6) {
+					addr_to_str(&l->rx.addr, addr, sizeof(addr));
+					port_to_str(&l->rx.addr, port, sizeof(port));
 					chunk_appendf(trash, "ipv6@[%s]:%s", addr, port);
-				} else if (l->addr.ss_family == AF_CUST_SOCKPAIR) {
-					chunk_appendf(trash, "sockpair@%d", ((struct sockaddr_in *)&l->addr)->sin_addr.s_addr);
+				} else if (l->rx.addr.ss_family == AF_CUST_SOCKPAIR) {
+					chunk_appendf(trash, "sockpair@%d", ((struct sockaddr_in *)&l->rx.addr)->sin_addr.s_addr);
 				}
 			}
 		}
@@ -530,6 +532,22 @@ static int cli_parse_request(struct appctx *appctx)
 	p = appctx->chunk->area;
 	end = p + appctx->chunk->data;
 
+	/*
+	 * Get the payload start if there is one.
+	 * For the sake of simplicity, the payload pattern is looked up
+	 * everywhere from the start of the input but it can only be found
+	 * at the end of the first line if APPCTX_CLI_ST1_PAYLOAD is set.
+	 *
+	 * The input string was zero terminated so it is safe to use
+	 * the str*() functions throughout the parsing
+	 */
+	if (appctx->st1 & APPCTX_CLI_ST1_PAYLOAD) {
+		payload = strstr(p, PAYLOAD_PATTERN);
+		end = payload;
+		/* skip the pattern */
+		payload += strlen(PAYLOAD_PATTERN);
+	}
+
 	/*
 	 * Get pointers on words.
 	 * One extra slot is reserved to store a pointer on a null byte.
@@ -542,15 +560,6 @@ static int cli_parse_request(struct appctx *appctx)
 		if (!*p)
 			break;
 
-		if (strcmp(p, PAYLOAD_PATTERN) == 0) {
-			/* payload pattern recognized here, this is not an arg anymore,
-			 * the payload starts at the first byte that follows the zero
-			 * after the pattern.
-			 */
-			payload = p + strlen(PAYLOAD_PATTERN) + 1;
-			break;
-		}
-
 		args[i] = p;
 		while (1) {
 			p += strcspn(p, " \t\\");
@@ -591,11 +600,11 @@ static int cli_parse_request(struct appctx *appctx)
 		return 0;
 
 	/* in a worker or normal process, don't display master only commands */
-	if (master == 0 && (kw->level & ACCESS_MASTER_ONLY))
+	if (appctx->applet == &cli_applet && (kw->level & ACCESS_MASTER_ONLY))
 		return 0;
 
 	/* in master don't displays if we don't have the master bits */
-	if (master == 1 && !(kw->level & (ACCESS_MASTER_ONLY|ACCESS_MASTER)))
+	if (appctx->applet == &mcli_applet && !(kw->level & (ACCESS_MASTER_ONLY|ACCESS_MASTER)))
 		return 0;
 
 	/* only accept expert commands in expert mode */
@@ -695,7 +704,6 @@ static void cli_io_handler(struct appctx *appctx)
 			 */
 			si_shutw(si);
 			free_trash_chunk(appctx->chunk);
-			appctx->chunk = NULL;
 			break;
 		}
 		else if (appctx->st0 == CLI_ST_GETREQ) {
@@ -720,20 +728,9 @@ static void cli_io_handler(struct appctx *appctx)
 				break;
 			}
 
-			/* payload doesn't take escapes nor does it end on semi-colons, so
-			 * we use the regular getline. Normal mode however must stop on
-			 * LFs and semi-colons that are not prefixed by a backslash. Note
-			 * that we reserve one byte at the end to insert a trailing nul byte.
-			 */
-
-			if (appctx->st1 & APPCTX_CLI_ST1_PAYLOAD)
-				reql = co_getline(si_oc(si), str,
-				                  appctx->chunk->size - appctx->chunk->data - 1);
-			else
-				reql = co_getdelim(si_oc(si), str,
-				                   appctx->chunk->size - appctx->chunk->data - 1,
-				                   "\n;", '\\');
-
+			/* '- 1' is to ensure a null byte can always be inserted at the end */
+			reql = co_getline(si_oc(si), str,
+					  appctx->chunk->size - appctx->chunk->data - 1);
 			if (reql <= 0) { /* closed or EOL not found */
 				if (reql == 0)
 					break;
@@ -806,10 +803,8 @@ static void cli_io_handler(struct appctx *appctx)
 				 * Its location is not remembered here, this is just to switch
 				 * to a gathering mode.
 				 */
-				if (strcmp(appctx->chunk->area + appctx->chunk->data - strlen(PAYLOAD_PATTERN), PAYLOAD_PATTERN) == 0) {
+				if (!strcmp(appctx->chunk->area + appctx->chunk->data - strlen(PAYLOAD_PATTERN), PAYLOAD_PATTERN))
 					appctx->st1 |= APPCTX_CLI_ST1_PAYLOAD;
-					appctx->chunk->data++; // keep the trailing \0 after '<<'
-				}
 				else {
 					/* no payload, the command is complete: parse the request */
 					if (!cli_parse_request(appctx))
@@ -924,17 +919,6 @@ static void cli_io_handler(struct appctx *appctx)
 			appctx->st0 = CLI_ST_GETREQ;
 			/* reactivate the \n at the end of the response for the next command */
 			appctx->st1 &= ~APPCTX_CLI_ST1_NOLF;
-
-			/* this forces us to yield between pipelined commands and
-			 * avoid extremely long latencies (e.g. "del map" etc). In
-			 * addition this increases the likelihood that the stream
-			 * refills the buffer with new bytes in non-interactive
-			 * mode, avoiding to close on apparently empty commands.
-			 */
-			if (co_data(si_oc(si))) {
-				appctx_wakeup(appctx);
-				goto out;
-			}
 		}
 	}
 
@@ -972,9 +956,6 @@ static void cli_io_handler(struct appctx *appctx)
  */
 static void cli_release_handler(struct appctx *appctx)
 {
-	free_trash_chunk(appctx->chunk);
-	appctx->chunk = NULL;
-
 	if (appctx->io_release) {
 		appctx->io_release(appctx);
 		appctx->io_release = NULL;
@@ -1085,7 +1066,7 @@ static int cli_io_handler_show_fd(struct appctx *appctx)
 			if (conn->handle.fd != fd)
 				suspicious = 1;
 		}
-		else if (fdt.iocb == listener_accept)
+		else if (fdt.iocb == sock_accept_iocb)
 			li = fdt.owner;
 
 		if (!fdt.thread_mask)
@@ -1147,7 +1128,7 @@ static int cli_io_handler_show_fd(struct appctx *appctx)
 			if (px)
 				chunk_appendf(&trash, " px=%s", px->id);
 			else if (sv)
-				chunk_appendf(&trash, " sv=%s/%s", sv->proxy->id, sv->id);
+				chunk_appendf(&trash, " sv=%s/%s", sv->id, sv->proxy->id);
 			else if (li)
 				chunk_appendf(&trash, " fe=%s", li->bind_conf->frontend->id);
 
@@ -1169,7 +1150,7 @@ static int cli_io_handler_show_fd(struct appctx *appctx)
 					suspicious |= xprt->show_fd(&trash, conn, xprt_ctx);
 			}
 		}
-		else if (fdt.iocb == listener_accept) {
+		else if (fdt.iocb == sock_accept_iocb) {
 			struct sockaddr_storage sa;
 			socklen_t salen;
 
@@ -1239,10 +1220,8 @@ static int cli_io_handler_show_activity(struct appctx *appctx)
 		unsigned int _v[MAX_THREADS];				\
 		unsigned int _tot;					\
 		const unsigned int _nbt = global.nbthread;		\
-		_tot = t = 0;						\
-		do {							\
+		for (_tot = t = 0; t < _nbt; t++)			\
 			_tot += _v[t] = (x);				\
-		} while (++t < _nbt);					\
 		if (_nbt == 1) {					\
 			chunk_appendf(&trash, " %u\n", _tot);		\
 			break;						\
@@ -1259,10 +1238,8 @@ static int cli_io_handler_show_activity(struct appctx *appctx)
 		unsigned int _v[MAX_THREADS];				\
 		unsigned int _tot;					\
 		const unsigned int _nbt = global.nbthread;		\
-		_tot = t = 0;						\
-		do {							\
+		for (_tot = t = 0; t < _nbt; t++)			\
 			_tot += _v[t] = (x);				\
-		} while (++t < _nbt);					\
 		if (_nbt == 1) {					\
 			chunk_appendf(&trash, " %u\n", _tot);		\
 			break;						\
@@ -1365,25 +1342,25 @@ static int cli_io_handler_show_cli_sock(struct appctx *appctx)
 						char addr[46];
 						char port[6];
 
-						if (l->addr.ss_family == AF_UNIX) {
+						if (l->rx.addr.ss_family == AF_UNIX) {
 							const struct sockaddr_un *un;
 
-							un = (struct sockaddr_un *)&l->addr;
+							un = (struct sockaddr_un *)&l->rx.addr;
 							if (un->sun_path[0] == '\0') {
 								chunk_appendf(&trash, "abns@%s ", un->sun_path+1);
 							} else {
 								chunk_appendf(&trash, "unix@%s ", un->sun_path);
 							}
-						} else if (l->addr.ss_family == AF_INET) {
-							addr_to_str(&l->addr, addr, sizeof(addr));
-							port_to_str(&l->addr, port, sizeof(port));
+						} else if (l->rx.addr.ss_family == AF_INET) {
+							addr_to_str(&l->rx.addr, addr, sizeof(addr));
+							port_to_str(&l->rx.addr, port, sizeof(port));
 							chunk_appendf(&trash, "ipv4@%s:%s ", addr, port);
-						} else if (l->addr.ss_family == AF_INET6) {
-							addr_to_str(&l->addr, addr, sizeof(addr));
-							port_to_str(&l->addr, port, sizeof(port));
+						} else if (l->rx.addr.ss_family == AF_INET6) {
+							addr_to_str(&l->rx.addr, addr, sizeof(addr));
+							port_to_str(&l->rx.addr, port, sizeof(port));
 							chunk_appendf(&trash, "ipv6@[%s]:%s ", addr, port);
-						} else if (l->addr.ss_family == AF_CUST_SOCKPAIR) {
-							chunk_appendf(&trash, "sockpair@%d ", ((struct sockaddr_in *)&l->addr)->sin_addr.s_addr);
+						} else if (l->rx.addr.ss_family == AF_CUST_SOCKPAIR) {
+							chunk_appendf(&trash, "sockpair@%d ", ((struct sockaddr_in *)&l->rx.addr)->sin_addr.s_addr);
 						} else
 							chunk_appendf(&trash, "unknown ");
 
@@ -1396,10 +1373,10 @@ static int cli_io_handler_show_cli_sock(struct appctx *appctx)
 						else
 							chunk_appendf(&trash, "  ");
 
-						if (bind_conf->bind_proc != 0) {
+						if (bind_conf->settings.bind_proc != 0) {
 							int pos;
-							for (pos = 0; pos < 8 * sizeof(bind_conf->bind_proc); pos++) {
-								if (bind_conf->bind_proc & (1UL << pos)) {
+							for (pos = 0; pos < 8 * sizeof(bind_conf->settings.bind_proc); pos++) {
+								if (bind_conf->settings.bind_proc & (1UL << pos)) {
 									chunk_appendf(&trash, "%d,", pos+1);
 								}
 							}
@@ -1610,13 +1587,6 @@ static int cli_parse_expert_mode(char **args, char *payload, struct appctx *appc
 	return 1;
 }
 
-/* shows HAProxy version */
-static int cli_parse_show_version(char **args, char *payload, struct appctx *appctx, void *private)
-{
-	char *msg = NULL;
-
-	return cli_dynmsg(appctx, LOG_INFO, memprintf(&msg, "%s\n", haproxy_version));
-}
 
 int cli_parse_default(char **args, char *payload, struct appctx *appctx, void *private)
 {
@@ -1731,81 +1701,6 @@ static int bind_parse_severity_output(char **args, int cur_arg, struct proxy *px
 	}
 }
 
-/*
- * For one proxy, fill the iov and send the msghdr. Also update fd_it and offset.
- * Return -1 upon error, otherwise 0.
- *
- * This function is only meant to deduplicate the code between the peers and
- * the proxy list in _getsocks(), not to be used anywhere else.
- */
-inline static int _getsocks_gen_send(struct proxy *px, int sendfd, int *tmpfd, struct iovec *iov,
-                                     int tot_fd_nb, int *fd_it, int *offset, struct msghdr *msghdr)
-{
-	int i = *fd_it;
-	int curoff = *offset;
-	struct listener *l;
-	unsigned char *tmpbuf = iov->iov_base;
-
-	list_for_each_entry(l, &px->conf.listeners, by_fe) {
-		int ret;
-		/* Only transfer IPv4/IPv6 sockets */
-		if (l->state >= LI_ZOMBIE &&
-		    (l->proto->sock_family == AF_INET ||
-		     l->proto->sock_family == AF_INET6 ||
-		     l->proto->sock_family == AF_UNIX)) {
-			memcpy(&tmpfd[i % MAX_SEND_FD], &l->fd, sizeof(l->fd));
-			if (!l->netns)
-				tmpbuf[curoff++] = 0;
-#ifdef USE_NS
-			else {
-				char *name = l->netns->node.key;
-				unsigned char len = l->netns->name_len;
-				tmpbuf[curoff++] = len;
-				memcpy(tmpbuf + curoff, name, len);
-				curoff += len;
-			}
-#endif
-			if (l->interface) {
-				unsigned char len = strlen(l->interface);
-				tmpbuf[curoff++] = len;
-				memcpy(tmpbuf + curoff, l->interface, len);
-				curoff += len;
-			} else
-				tmpbuf[curoff++] = 0;
-			memcpy(tmpbuf + curoff, &l->options,
-			       sizeof(l->options));
-			curoff += sizeof(l->options);
-
-			i++;
-		} else
-			continue;
-		/* if it reaches the max number of fd per msghdr */
-		if ((!(i % MAX_SEND_FD))) {
-			iov->iov_len = curoff;
-			if (sendmsg(sendfd, msghdr, 0) != curoff) {
-				ha_warning("Failed to transfer sockets\n");
-				return -1;
-			}
-			/* Wait for an ack */
-			do {
-				ret = recv(sendfd, &tot_fd_nb,
-				           sizeof(tot_fd_nb), 0);
-			} while (ret == -1 && errno == EINTR);
-			if (ret <= 0) {
-				ha_warning("Unexpected error while transferring sockets\n");
-				return -1;
-			}
-			curoff = 0;
-		}
-	}
-
-	*fd_it = i;
-	*offset = curoff;
-
-	return 0;
-}
-
-
 /* Send all the bound sockets, always returns 1 */
 static int _getsocks(char **args, char *payload, struct appctx *appctx, void *private)
 {
@@ -1818,11 +1713,12 @@ static int _getsocks(char **args, char *payload, struct appctx *appctx, void *pr
 	struct msghdr msghdr;
 	struct iovec iov;
 	struct timeval tv = { .tv_sec = 1, .tv_usec = 0 };
+	const char *ns_name, *if_name;
+	unsigned char ns_nlen, if_nlen;
+	int nb_queued;
+	int cur_fd = 0;
 	int *tmpfd;
 	int tot_fd_nb = 0;
-	struct proxy *px;
-	struct peers *prs;
-	int i = 0;
 	int fd = -1;
 	int curoff = 0;
 	int old_fcntl = -1;
@@ -1858,38 +1754,11 @@ static int _getsocks(char **args, char *payload, struct appctx *appctx, void *pr
 	memset(&msghdr, 0, sizeof(msghdr));
 	/*
 	 * First, calculates the total number of FD, so that we can let
-	 * the caller know how much he should expects.
+	 * the caller know how much it should expect.
 	 */
-	px = proxies_list;
-	while (px) {
-		struct listener *l;
+	for (cur_fd = 0;cur_fd < global.maxsock; cur_fd++)
+		tot_fd_nb += fdtab[cur_fd].exported;
 
-		list_for_each_entry(l, &px->conf.listeners, by_fe) {
-			/* Only transfer IPv4/IPv6/UNIX sockets */
-			if (l->state >= LI_ZOMBIE &&
-			    (l->proto->sock_family == AF_INET ||
-			    l->proto->sock_family == AF_INET6 ||
-			    l->proto->sock_family == AF_UNIX))
-				tot_fd_nb++;
-		}
-		px = px->next;
-	}
-	prs = cfg_peers;
-	while (prs) {
-		if (prs->peers_fe) {
-			struct listener *l;
-
-			list_for_each_entry(l, &prs->peers_fe->conf.listeners, by_fe) {
-				/* Only transfer IPv4/IPv6/UNIX sockets */
-				if (l->state >= LI_ZOMBIE &&
-				    (l->proto->sock_family == AF_INET ||
-				     l->proto->sock_family == AF_INET6 ||
-				     l->proto->sock_family == AF_UNIX))
-					tot_fd_nb++;
-			}
-		}
-		prs = prs->next;
-	}
 	if (tot_fd_nb == 0)
 		goto out;
 
@@ -1918,7 +1787,7 @@ static int _getsocks(char **args, char *payload, struct appctx *appctx, void *pr
 	 *  The namespace name, if any
 	 *  Size of the interface name (or 0 if none), as an unsigned char
 	 *  The interface name, if any
-	 *  Listener options, as an int.
+	 *  32 bits of zeroes (used to be listener options).
 	 */
 	/* We will send sockets MAX_SEND_FD per MAX_SEND_FD, allocate a
 	 * buffer big enough to store the socket information.
@@ -1928,28 +1797,81 @@ static int _getsocks(char **args, char *payload, struct appctx *appctx, void *pr
 		ha_warning("Failed to allocate memory to transfer socket information\n");
 		goto out;
 	}
+
+	nb_queued = 0;
 	iov.iov_base = tmpbuf;
-	px = proxies_list;
-	while (px) {
-		if (_getsocks_gen_send(px, fd, tmpfd, &iov,
-		                   tot_fd_nb, &i, &curoff, &msghdr) < 0)
-			goto out;
-		px = px->next;
-	}
-	/* should be done for peers too */
-	prs = cfg_peers;
-	while (prs) {
-		if (prs->peers_fe)
-			if (_getsocks_gen_send(prs->peers_fe, fd, tmpfd, &iov,
-			                       tot_fd_nb, &i, &curoff, &msghdr) < 0)
-				goto out;
-		prs = prs->next;
+	for (cur_fd = 0; cur_fd < global.maxsock; cur_fd++) {
+		if (!(fdtab[cur_fd].exported))
+			continue;
+
+		ns_name = if_name = "";
+		ns_nlen = if_nlen = 0;
+
+		/* for now we can only retrieve namespaces and interfaces from
+		 * pure listeners.
+		 */
+		if (fdtab[cur_fd].iocb == sock_accept_iocb) {
+			const struct listener *l = fdtab[cur_fd].owner;
+
+			if (l->rx.settings->interface) {
+				if_name = l->rx.settings->interface;
+				if_nlen = strlen(if_name);
+			}
+
+#ifdef USE_NS
+			if (l->rx.settings->netns) {
+				ns_name = l->rx.settings->netns->node.key;
+				ns_nlen = l->rx.settings->netns->name_len;
+			}
+#endif
+		}
+
+		/* put the FD into the CMSG_DATA */
+		tmpfd[nb_queued++] = cur_fd;
+
+		/* first block is <ns_name_len> <ns_name> */
+		tmpbuf[curoff++] = ns_nlen;
+		if (ns_nlen)
+			memcpy(tmpbuf + curoff, ns_name, ns_nlen);
+		curoff += ns_nlen;
+
+		/* second block is <if_name_len> <if_name> */
+		tmpbuf[curoff++] = if_nlen;
+		if (if_nlen)
+			memcpy(tmpbuf + curoff, if_name, if_nlen);
+		curoff += if_nlen;
+
+		/* we used to send the listener options here before 2.3 */
+		memset(tmpbuf + curoff, 0, sizeof(int));
+		curoff += sizeof(int);
+
+		/* there's a limit to how many FDs may be sent at once */
+		if (nb_queued == MAX_SEND_FD) {
+			iov.iov_len = curoff;
+			if (sendmsg(fd, &msghdr, 0) != curoff) {
+				ha_warning("Failed to transfer sockets\n");
+				return -1;
+			}
+
+			/* Wait for an ack */
+			do {
+				ret = recv(fd, &tot_fd_nb, sizeof(tot_fd_nb), 0);
+			} while (ret == -1 && errno == EINTR);
+
+			if (ret <= 0) {
+				ha_warning("Unexpected error while transferring sockets\n");
+				return -1;
+			}
+			curoff = 0;
+			nb_queued = 0;
+		}
 	}
 
-	if (i % MAX_SEND_FD) {
+	/* flush pending stuff */
+	if (nb_queued) {
 		iov.iov_len = curoff;
-		cmsg->cmsg_len = CMSG_LEN((i % MAX_SEND_FD) * sizeof(int));
-		msghdr.msg_controllen = CMSG_SPACE(sizeof(int) *  (i % MAX_SEND_FD));
+		cmsg->cmsg_len = CMSG_LEN(nb_queued * sizeof(int));
+		msghdr.msg_controllen = CMSG_SPACE(nb_queued * sizeof(int));
 		if (sendmsg(fd, &msghdr, 0) != curoff) {
 			ha_warning("Failed to transfer sockets\n");
 			goto out;
@@ -2017,9 +1939,9 @@ static enum obj_type *pcli_pid_to_server(int proc_pid)
 {
 	struct mworker_proc *child;
 
-	/* return the CLI applet of the master */
+	/* return the  mCLI applet of the master */
 	if (proc_pid == 0)
-		return &cli_applet.obj_type;
+		return &mcli_applet.obj_type;
 
 	list_for_each_entry(child, &proc_list, list) {
 		if (child->pid == proc_pid){
@@ -2160,8 +2082,8 @@ int pcli_find_and_exec_kw(struct stream *s, char **args, int argl, char **errmsg
  */
 int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int *next_pid)
 {
-	char *str;
-	char *end;
+	char *str = (char *)ci_head(req);
+	char *end = (char *)ci_stop(req);
 	char *args[MAX_STATS_ARGS + 1]; /* +1 for storing a NULL */
 	int argl; /* number of args */
 	char *p;
@@ -2172,15 +2094,6 @@ int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int
 	int ret;
 	int i = 0;
 
-	/* we cannot deal with a wrapping buffer, so let's take care of this
-	 * first.
-	 */
-	if (b_head(&req->buf) + b_data(&req->buf) > b_wrap(&req->buf))
-		b_slow_realign(&req->buf, trash.area, co_data(req));
-
-	str = (char *)ci_head(req);
-	end = (char *)ci_stop(req);
-
 	p = str;
 
 	if (!(s->pcli_flags & PCLI_F_PAYLOAD)) {
@@ -2215,7 +2128,7 @@ int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int
 	end = p + reql;
 
 	/* there is no end to this command, need more to parse ! */
-	if (!reql || *(end-1) != '\n') {
+	if (*(end-1) != '\n') {
 		return -1;
 	}
 
@@ -2314,13 +2227,6 @@ int pcli_wait_for_request(struct stream *s, struct channel *req, int an_bit)
 	int to_forward;
 	char *errmsg = NULL;
 
-	/* Don't read the next command if still processing the reponse of the
-	 * current one. Just wait. At this stage, errors should be handled by
-	 * the response analyzer.
-	 */
-	if (s->res.analysers & AN_RES_WAIT_CLI)
-		return 0;
-
 	if ((s->pcli_flags & ACCESS_LVL_MASK) == ACCESS_LVL_NONE)
 		s->pcli_flags |= strm_li(s)->bind_conf->level & ACCESS_LVL_MASK;
 
@@ -2333,11 +2239,20 @@ read_again:
 	/* We don't know yet to which server we will connect */
 	channel_dont_connect(req);
 
+
+	/* we are not waiting for a response, there is no more request and we
+	 * receive a close from the client, we can leave */
+	if (!(ci_data(req)) && req->flags & CF_SHUTR) {
+		channel_shutw_now(&s->res);
+		s->req.analysers &= ~AN_REQ_WAIT_CLI;
+		return 1;
+	}
+
 	req->flags |= CF_READ_DONTWAIT;
 
 	/* need more data */
 	if (!ci_data(req))
-		goto missing_data;
+		return 0;
 
 	/* If there is data available for analysis, log the end of the idle time. */
 	if (c_data(req) && s->logs.t_idle == -1)
@@ -2359,6 +2274,13 @@ read_again:
 		}
 
 		s->res.flags |= CF_WAKE_ONCE; /* need to be called again */
+
+		/* remove the XFER_DATA analysers, which forwards all
+		 * the data, we don't want to forward the next requests
+		 * We need to add CF_FLT_ANALYZE to abort the forward too.
+		 */
+		req->analysers &= ~(AN_REQ_FLT_XFER_DATA|AN_REQ_WAIT_CLI);
+		req->analysers |= AN_REQ_FLT_END|CF_FLT_ANALYZE;
 		s->res.analysers |= AN_RES_WAIT_CLI;
 
 		if (!(s->flags & SF_ASSIGNED)) {
@@ -2369,9 +2291,6 @@ read_again:
 			/* we can connect now */
 			s->target = pcli_pid_to_server(target_pid);
 
-			if (!s->target)
-				goto server_disconnect;
-
 			s->flags |= (SF_DIRECT | SF_ASSIGNED);
 			channel_auto_connect(req);
 		}
@@ -2380,14 +2299,13 @@ read_again:
 		/* we trimmed things but we might have other commands to consume */
 		pcli_write_prompt(s);
 		goto read_again;
-	} else if (to_forward == -1) {
-                if (errmsg) {
-                        /* there was an error during the parsing */
-                        pcli_reply_and_close(s, errmsg);
-                        s->req.analysers &= ~AN_REQ_WAIT_CLI;
-                        return 0;
-                }
-                goto missing_data;
+	} else if (to_forward == -1 && errmsg) {
+		/* there was an error during the parsing */
+			pcli_reply_and_close(s, errmsg);
+			return 0;
+	} else if (to_forward == -1 && channel_full(req, global.tune.maxrewrite)) {
+		/* buffer is full and we didn't catch the end of a command */
+		goto send_help;
 	}
 
 	return 0;
@@ -2396,24 +2314,6 @@ send_help:
 	b_reset(&req->buf);
 	b_putblk(&req->buf, "help\n", 5);
 	goto read_again;
-
-missing_data:
-        if (req->flags & CF_SHUTR) {
-                /* There is no more request or a only a partial one and we
-                 * receive a close from the client, we can leave */
-                channel_shutw_now(&s->res);
-                s->req.analysers &= ~AN_REQ_WAIT_CLI;
-                return 1;
-        }
-        else if (channel_full(req, global.tune.maxrewrite)) {
-                /* buffer is full and we didn't catch the end of a command */
-                goto send_help;
-        }
-        return 0;
-
-server_disconnect:
-	pcli_reply_and_close(s, "Can't connect to the target CLI!\n");
-	return 0;
 }
 
 int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
@@ -2421,10 +2321,8 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 	struct proxy *fe = strm_fe(s);
 	struct proxy *be = s->be;
 
-	if ((rep->flags & (CF_READ_ERROR|CF_READ_TIMEOUT|CF_WRITE_ERROR|CF_WRITE_TIMEOUT)) ||
-	    ((rep->flags & CF_SHUTW) && (rep->to_forward || co_data(rep)))) {
+	if (rep->flags & CF_READ_ERROR) {
 		pcli_reply_and_close(s, "Can't connect to the target CLI!\n");
-		s->req.analysers &= ~AN_REQ_WAIT_CLI;
 		s->res.analysers &= ~AN_RES_WAIT_CLI;
 		return 0;
 	}
@@ -2436,6 +2334,7 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 	channel_dont_close(&s->req);
 
 	if (s->pcli_flags & PCLI_F_PAYLOAD) {
+		s->req.analysers |= AN_REQ_WAIT_CLI;
 		s->res.analysers &= ~AN_RES_WAIT_CLI;
 		s->req.flags |= CF_WAKE_ONCE; /* need to be called again if there is some command left in the request */
 		return 0;
@@ -2513,7 +2412,7 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 				HA_ATOMIC_SUB(&__objt_server(s->target)->cur_sess, 1);
 			}
 			if (may_dequeue_tasks(__objt_server(s->target), be))
-				process_srv_queue(__objt_server(s->target), 0);
+				process_srv_queue(__objt_server(s->target));
 		}
 
 		s->target = NULL;
@@ -2556,6 +2455,7 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 
 		s->req.flags |= CF_WAKE_ONCE; /* need to be called again if there is some command left in the request */
 
+		s->req.analysers |= AN_REQ_WAIT_CLI;
 		s->res.analysers &= ~AN_RES_WAIT_CLI;
 
 		/* We must trim any excess data from the response buffer, because we
@@ -2631,7 +2531,6 @@ int mworker_cli_proxy_create()
 	proxies_list = mworker_proxy;
 	mworker_proxy->id = strdup("MASTER");
 	mworker_proxy->mode = PR_MODE_CLI;
-	mworker_proxy->state = PR_STNEW;
 	mworker_proxy->last_change = now.tv_sec;
 	mworker_proxy->cap = PR_CAP_LISTEN; /* this is a listen section */
 	mworker_proxy->maxconn = 10;                 /* default to 10 concurrent connections */
@@ -2678,14 +2577,14 @@ int mworker_cli_proxy_create()
 		newsrv->conf.line = 0;
 
 		memprintf(&msg, "sockpair@%d", child->ipc_fd[0]);
-		if ((sk = str2sa_range(msg, &port, &port1, &port2, &errmsg, NULL, NULL, 0)) == 0) {
+		if ((sk = str2sa_range(msg, &port, &port1, &port2, NULL, &proto,
+		                       &errmsg, NULL, NULL, PA_O_STREAM)) == 0) {
 			goto error;
 		}
 		free(msg);
 		msg = NULL;
 
-		proto = protocol_by_family(sk->ss_family);
-		if (!proto || !proto->connect) {
+		if (!proto->connect) {
 			goto error;
 		}
 
@@ -2805,7 +2704,8 @@ int mworker_cli_proxy_new_listener(char *line)
 		l->accept = session_accept_fd;
 		l->default_target = mworker_proxy->default_target;
 		/* don't make the peers subject to global limits and don't close it in the master */
-		l->options |= (LI_O_UNLIMITED|LI_O_MWORKER); /* we are keeping this FD in the master */
+		l->options  |= LI_O_UNLIMITED;
+		l->rx.flags |= RX_F_MWORKER; /* we are keeping this FD in the master */
 		l->nice = -64;  /* we want to boost priority for local stats */
 		global.maxsock++; /* for the listening socket */
 	}
@@ -2853,7 +2753,7 @@ int mworker_cli_sockpair_new(struct mworker_proc *mworker_proc, int proc)
 	bind_conf->level &= ~ACCESS_LVL_MASK;
 	bind_conf->level |= ACCESS_LVL_ADMIN; /* TODO: need to lower the rights with a CLI keyword*/
 
-	bind_conf->bind_proc = 1UL << proc;
+	bind_conf->settings.bind_proc = 1UL << proc;
 	global.stats_fe->bind_proc = 0; /* XXX: we should be careful with that, it can be removed by configuration */
 
 	if (!memprintf(&path, "sockpair@%d", mworker_proc->ipc_fd[1])) {
@@ -2873,8 +2773,9 @@ int mworker_cli_sockpair_new(struct mworker_proc *mworker_proc, int proc)
 		l->accept = session_accept_fd;
 		l->default_target = global.stats_fe->default_target;
 		l->options |= (LI_O_UNLIMITED | LI_O_NOSTOP);
+		HA_ATOMIC_ADD(&unstoppable_jobs, 1);
 		/* it's a sockpair but we don't want to keep the fd in the master */
-		l->options &= ~LI_O_INHERITED;
+		l->rx.flags &= ~RX_F_INHERITED;
 		l->nice = -64;  /* we want to boost priority for local stats */
 		global.maxsock++; /* for the listening socket */
 	}
@@ -2896,6 +2797,14 @@ static struct applet cli_applet = {
 	.release = cli_release_handler,
 };
 
+/* master CLI */
+static struct applet mcli_applet = {
+	.obj_type = OBJ_TYPE_APPLET,
+	.name = "<MCLI>", /* used for logging */
+	.fct = cli_io_handler,
+	.release = cli_release_handler,
+};
+
 /* register cli keywords */
 static struct cli_kw_list cli_kws = {{ },{
 	{ { "help", NULL }, NULL, cli_parse_simple, NULL },
@@ -2910,7 +2819,6 @@ static struct cli_kw_list cli_kws = {{ },{
 	{ { "show", "cli", "level", NULL },    "show cli level   : display the level of the current CLI session", cli_parse_show_lvl, NULL, NULL, NULL, ACCESS_MASTER},
 	{ { "show", "fd", NULL }, "show fd [num] : dump list of file descriptors in use", cli_parse_show_fd, cli_io_handler_show_fd, NULL },
 	{ { "show", "activity", NULL }, "show activity : show per-thread activity stats (for support/developers)", cli_parse_default, cli_io_handler_show_activity, NULL },
-	{ { "show", "version", NULL },  "show version  : show version of the current process", cli_parse_show_version, NULL, NULL, NULL, ACCESS_MASTER },
 	{ { "operator", NULL },  "operator       : lower the level of the current CLI session to operator", cli_parse_set_lvl, NULL, NULL, NULL, ACCESS_MASTER},
 	{ { "user", NULL },      "user           : lower the level of the current CLI session to user", cli_parse_set_lvl, NULL, NULL, NULL, ACCESS_MASTER},
 	{ { "_getsocks", NULL }, NULL,  _getsocks, NULL },
