@@ -22,18 +22,20 @@
 
 struct pool_head *pool_head_buffer;
 
-/* list of objects waiting for at least one buffer */
-struct mt_list buffer_wq = LIST_HEAD_INIT(buffer_wq);
-
 /* perform minimal intializations, report 0 in case of error, 1 if OK. */
 int init_buffer()
 {
 	void *buffer;
+	int thr;
 
 	pool_head_buffer = create_pool("buffer", global.tune.bufsize, MEM_F_SHARED|MEM_F_EXACT);
 	if (!pool_head_buffer)
 		return 0;
 
+	for (thr = 0; thr < MAX_THREADS; thr++)
+		LIST_INIT(&ha_thread_info[thr].buffer_wq);
+
+
 	/* The reserved buffer is what we leave behind us. Thus we always need
 	 * at least one extra buffer in minavail otherwise we'll end up waking
 	 * up tasks with no memory available, causing a lot of useless wakeups.
@@ -94,33 +96,26 @@ void buffer_dump(FILE *o, struct buffer *b, int from, int to)
 	fflush(o);
 }
 
-/* see offer_buffer() for details */
-void __offer_buffer(void *from, unsigned int threshold)
+/* see offer_buffers() for details */
+void __offer_buffers(void *from, unsigned int count)
 {
-	struct buffer_wait *wait;
-	struct mt_list *elt1, elt2;
-	int avail;
+	struct buffer_wait *wait, *wait_back;
 
 	/* For now, we consider that all objects need 1 buffer, so we can stop
 	 * waking up them once we have enough of them to eat all the available
 	 * buffers. Note that we don't really know if they are streams or just
 	 * other tasks, but that's a rough estimate. Similarly, for each cached
-	 * event we'll need 1 buffer. If no buffer is currently used, always
-	 * wake up the number of tasks we can offer a buffer based on what is
-	 * allocated, and in any case at least one task per two reserved
-	 * buffers.
+	 * event we'll need 1 buffer.
 	 */
-	avail = pool_head_buffer->allocated - pool_head_buffer->used - global.tune.reserved_bufs / 2;
-
-	mt_list_for_each_entry_safe(wait, &buffer_wq, list, elt1, elt2) {
-		if (avail <= threshold)
+	list_for_each_entry_safe(wait, wait_back, &ti->buffer_wq, list) {
+		if (!count)
 			break;
 
 		if (wait->target == from || !wait->wakeup_cb(wait->target))
 			continue;
 
-		MT_LIST_DEL_SAFE(elt1);
-		avail--;
+		LIST_DEL_INIT(&wait->list);
+		count--;
 	}
 }
 
