@@ -347,7 +347,7 @@ static int smp_fetch_meth(const struct arg *args, struct sample *smp, const char
 			return 0;
 		}
 
-		htx = smp_prefetch_htx(smp, chn, NULL, 0);
+		htx = smp_prefetch_htx(smp, chn, NULL, 1);
 		if (!htx)
 			return 0;
 
@@ -1015,7 +1015,7 @@ static int smp_fetch_hdr_ip(const struct arg *args, struct sample *smp, const ch
 			       smp->data.u.str.data);
 			temp->area[smp->data.u.str.data] = '\0';
 			len = url2ipv4((char *) temp->area, &smp->data.u.ipv4);
-			if (len == smp->data.u.str.data) {
+			if (len > 0 && len == smp->data.u.str.data) {
 				/* plain IPv4 address */
 				smp->data.type = SMP_T_IPV4;
 				break;
