@@ -53,8 +53,10 @@ static void mux_pt_destroy(struct mux_pt_ctx *ctx)
 	}
 }
 
-/* Callback, used when we get I/Os while in idle mode */
-static struct task *mux_pt_io_cb(struct task *t, void *tctx, unsigned short status)
+/* Callback, used when we get I/Os while in idle mode. This one is exported so
+ * that "show fd" can resolve it.
+ */
+struct task *mux_pt_io_cb(struct task *t, void *tctx, unsigned short status)
 {
 	struct mux_pt_ctx *ctx = tctx;
 
