@@ -1122,7 +1122,13 @@ struct task *manage_proxy(struct task *t, void *context, unsigned short state)
 	 * However we protect tables that are being synced to peers.
 	 */
 	if (unlikely(stopping && p->disabled && p->table && p->table->current)) {
-		if (!p->table->syncing) {
+
+		if (!p->table->refcnt) {
+			/* !table->refcnt means there
+			 * is no more pending full resync
+			 * to push to a new process and
+			 * we are free to flush the table.
+			 */
 			stktable_trash_oldest(p->table, p->table->current);
 			pool_gc(NULL);
 		}
@@ -1192,11 +1198,15 @@ struct task *hard_stop(struct task *t, void *context, unsigned short state)
 {
 	struct proxy *p;
 	struct stream *s;
+	int thr;
 
 	if (killed) {
 		ha_warning("Some tasks resisted to hard-stop, exiting now.\n");
 		send_log(NULL, LOG_WARNING, "Some tasks resisted to hard-stop, exiting now.\n");
 		killed = 2;
+		for (thr = 0; thr < global.nbthread; thr++)
+			if (((all_threads_mask & ~tid_bit) >> thr) & 1)
+				wake_thread(thr);
 		t->expire = TICK_ETERNITY;
 		return t;
 	}
@@ -1213,10 +1223,17 @@ struct task *hard_stop(struct task *t, void *context, unsigned short state)
 		}
 		p = p->next;
 	}
-	list_for_each_entry(s, &streams, list) {
-		stream_shutdown(s, SF_ERR_KILLED);
+
+	thread_isolate();
+
+	for (thr = 0; thr < global.nbthread; thr++) {
+		list_for_each_entry(s, &ha_thread_info[thr].streams, list) {
+			stream_shutdown(s, SF_ERR_KILLED);
+		}
 	}
 
+	thread_release();
+
 	killed = 1;
 	t->expire = tick_add(now_ms, MS_TO_TICKS(1000));
 	return t;
