@@ -1314,6 +1314,7 @@ static int cli_parse_del_crtlist(char **args, char *payload, struct appctx *appc
 	int linenum = 0;
 	char *colons;
 	char *end;
+	int error_message_dumped = 0;
 
 	if (!cli_has_level(appctx, ACCESS_LVL_ADMIN))
 		return 1;
@@ -1394,6 +1395,20 @@ static int cli_parse_del_crtlist(char **args, char *payload, struct appctx *appc
 		goto error;
 	}
 
+	/* Iterate over all the instances in order to see if any of them is a
+	 * default instance. If this is the case, the entry won't be suppressed. */
+	list_for_each_entry_safe(inst, inst_s, &entry->ckch_inst, by_crtlist_entry) {
+		if (inst->is_default && !inst->bind_conf->strict_sni) {
+			if (!error_message_dumped) {
+				memprintf(&err, "certificate '%s' cannot be deleted, it is used as default certificate by the following frontends:\n", cert_path);
+				error_message_dumped = 1;
+			}
+			memprintf(&err, "%s\t- %s:%d\n", err, inst->bind_conf->file, inst->bind_conf->line);
+		}
+	}
+	if (error_message_dumped)
+		goto error;
+
 	/* upon error free the ckch_inst and everything inside */
 
 	ebpt_delete(&entry->node);
